@@ -17,6 +17,7 @@ def dict2tuple(name,d):
            )
 
 class Dict2Data:
+    _req_keys = ()
     """
     - Returns a Data object with dictionary keys as attributes of Data accessible by dot notation or by key. Once an attribute is created, it can not be changed from outside.
     - **Parmeters**
@@ -39,16 +40,43 @@ class Dict2Data:
         > {'C': 2}
     """
     def __init__(self,d):
-        if isinstance(d,Dict2Data):
-            d = d.to_dict() # if nested Dict2Dataects, must expand here.
+        if not hasattr(self.__class__,'_req_keys'):
+            raise AttributeError("Derived class of `Dict2Data` should have attribute '_req_keys'")
+        if isinstance(d,(self.__class__, Dict2Data)):
+            d = d.to_dict() # if nested Dict2Data , must expand here.
+        # Check if all required keys are present in main level of subclasses
+        for key in self.__class__._req_keys:
+            if key not in d:
+                raise ValueError(f"Invalid input for {self.__class__.__name__}")
+        # ===================
         for a,b in d.items():
-            if isinstance(b,Dict2Data):
+            if isinstance(b,(self.__class__, Dict2Data)):
                 b = b.to_dict() # expands self instance !must here.
             if isinstance(b,(list,tuple)):
                 setattr(self,a,[Dict2Data(x) if isinstance(x,dict) else x for x in b])
             else:
                 setattr(self,a,Dict2Data(b) if isinstance(b,dict) else b)
-
+    
+    @classmethod
+    def validated(cls, data):
+        "Validate data like it's own or from json/pickle file/string."
+        if isinstance(data,cls):
+            return data
+        
+        if isinstance(data,(str,bytes)):
+            new_data = load(data, json_to = cls)
+            if not isinstance(new_data,cls):
+                raise TypeError(f"Data is not of type {cls}.")
+            return new_data
+        
+        if isinstance(data,Dict2Data) and cls is not Dict2Data: # Check for other classes strictly   
+            data_keys = data.keys()
+            for key in cls._req_keys:
+                if key not in data_keys:
+                    raise KeyError(f"Invalid data for {cls.__name__}")
+            
+        return cls(data) # make of that type at end
+        
     def to_dict(self):
         """Converts a `Dict2Data` object (root or nested level) to a dictionary.
         """
@@ -124,6 +152,7 @@ class Dict2Data:
         return self.__dict__.items()
     
 class VasprunData(Dict2Data):
+    _req_keys = ('kpath','bands','poscar')
     def __init__(self,d):
         super().__init__(d)
     
@@ -131,6 +160,7 @@ class VasprunData(Dict2Data):
         return super().__repr__().replace("Data","VasprunData",1)
     
 class SpinData(Dict2Data):
+    _req_keys = ('kpoints','spins','poscar')
     def __init__(self,d):
         super().__init__(d)
     
@@ -138,14 +168,25 @@ class SpinData(Dict2Data):
         return super().__repr__().replace("Data","SpinData",1)
     
 class PoscarData(Dict2Data):
+    _req_keys = ('basis','rec_basis','cartesian')
     def __init__(self,d):
-        super().__init__(self._add_text_plain(d))
+        if not isinstance(d,dict): # It could be other data type.
+            try:
+                d = d.to_dict()
+            except:
+                raise TypeError(f"{self.__class__} expects dict with valid keys or PoscarData got {d}")
+        fixed_dict = self._add_text_plain(d)
+        super().__init__(fixed_dict)
     
     def __repr__(self):
         return super().__repr__().replace("Data","PoscarData",1)
     
     def _add_text_plain(self, poscar_dict):
         "Returns poscar_dict with additional attribute `text_plain` after each operation"
+        for key in self.__class__._req_keys:
+            if key not in poscar_dict:
+                raise KeyError(f"Invalid input for PoscarData")
+
         comment = poscar_dict.get('text_plain',None) or '-- # Exported using pivotpy\n '
         comment = comment.splitlines()[0].split('#')
         comment = comment[1].strip() if comment[1:] else ''
@@ -162,6 +203,7 @@ class PoscarData(Dict2Data):
         return poscar_dict
 
 class LocpotData(Dict2Data):
+    _req_keys = ('ElemName','ElemIndex','SYSTEM')
     def __init__(self,d):
         super().__init__(d)
     
@@ -169,6 +211,7 @@ class LocpotData(Dict2Data):
         return super().__repr__().replace("Data","LocpotData",1)
 
 class OutcarData(Dict2Data):
+    _req_keys = ('site_pot','ion_pot','basis')
     def __init__(self,d):
         super().__init__(d)
     
